<template>
  <div class="chart-wraper">
    <div class="chart" ref="chartContainer"></div>
  </div>
</template>

<script setup>
import * as echarts from "echarts";
import "./theme/custom";

import { onMounted, ref, watchEffect, nextTick, onBeforeUnmount } from "vue";

const props = defineProps({
  unit: {
    type: String,
  },
});

const chartContainer = ref(null);
let myChart = null;
function getLinearColor(color0, color1) {
  return {
    type: "linear",
    x: 0,
    y: 0,
    x2: 0,
    y2: 1,
    colorStops: [
      {
        offset: 0,
        color: color0, // 0% 处的颜色
      },
      {
        offset: 1,
        color: color1, // 100% 处的颜色
      },
    ],
  };
}

function getOptions(xLabels = [], datas = [{ name: "" }, { name: "" }]) {
  let barColor = getLinearColor("#7bda59", "#059789");
  let maxColor = getLinearColor("#e3352b", "#f09f2e");
  let minColor = getLinearColor("#f09f2e", "#d3c209");
  let legends = [];
  datas.forEach((v, i) => {
    legends.push(v.name);
  });
  let data0 = datas[0].data || [];
  let data1 = datas[1].data || [];
  let data2 = data0.map((v, i) => v + data1[i]);
  return {
    grid: {
      left: 60,
      right: "1%",
    },
    legend: {
      x: "22%",
      top: "8%",
      textStyle: {
        color: "#ffffff",
      },
      data: legends,
    },
    tooltip: {
      trigger: "axis",
      axisPointer: {
        type: "shadow",
      },
    },
    xAxis: {
      type: "category",
      data: xLabels,
    },
    yAxis: {
      type: "value",
      name: props.unit ? `单位: ${props.unit}` : "",
      min: function (data) {
        let min = data.min;
        if (min == Infinity) {
          return 0;
        }
        return Number((min - min * 0.2).toFixed(2));
      },
      max: function (data) {
        let max = data.max;
        if (max == -Infinity) {
          max = 1;
        }
        return Number((max + max * 0.2).toFixed(2));
      },
    },
    series: [
      {
        name: legends[0],
        type: "bar",
        stack: "总量",
        barMaxWidth: 35,
        barGap: "10%",

        label: {
          show: true,
          color: "#fff",
          position: "inside",
          formatter: function (p) {
            return p.value > 0 ? p.value : "";
          },
        },
        itemStyle: {
          color: "rgba(255,144,128,1)",
        },
        data: data0,
        showBackground: true,
        backgroundStyle: {
          color: "rgba(0, 0, 0, 0.1)",
        },
      },

      {
        name: legends[1],
        type: "bar",
        stack: "总量",
        itemStyle: {
          color: "rgba(0,191,183,1)",
          borderRadius: 0,
        },
        label: {
          show: true,
          position: "inside",
          formatter: function (p) {
            return p.value > 0 ? p.value : "";
          },
        },
        data: data1,
        showBackground: true,
        backgroundStyle: {
          color: "rgba(0, 0, 0, 0.1)",
        },
      },
      {
        name: "总数",
        type: "line",
        itemStyle: {
          color: "transparent",
          borderRadius: 0,
        },
        label: {
          show: true,
          position: "top",
          formatter: function (p) {
            return p.value > 0 ? p.value : "";
          },
        },
        data: data2,
      },
    ],
    // series: [
    //   {
    //     data,
    //     type: "bar",
    //     barWidth: 20,
    //     itemStyle: {
    //       shape: "circle",
    //       borderRadius: [20, 20, 0, 0],
    //       color: (params) => {
    //         // console.log('params', params, data, '=============');
    //         let max = Math.max(...data);
    //         let min = Math.min(...data);
    //         let val = params.value;
    //         if (val == min) {
    //           return minColor;
    //         } else if (val == max) {
    //           return maxColor;
    //         }
    //         return barColor;
    //       },
    //     },
    //     showBackground: true,
    //     backgroundStyle: {
    //       color: "rgba(180, 180, 180, 0.2)",
    //     },
    //   },
    // ],
  };
}

function initChart() {
  if (chartContainer.value) {
    myChart = echarts.init(chartContainer.value, "custom");

    let option = getOptions();
    myChart.setOption(option);

    // 监听窗口变化重新渲染图表
    window.addEventListener("resize", () => {
      myChart.resize();
    });
  }
}

function updateChart(xLabels, datas) {
  let option = getOptions(xLabels, datas);
  myChart.setOption(option);
}

onMounted(() => {
  initChart();
});

watchEffect(() => {
  // 如果需要基于某些响应式数据更新图表，可以在这里处理
});

onBeforeUnmount(() => {
  if (myChart) {
    myChart.dispose();
  }
});

defineExpose({
  updateChart,
});
</script>

<style scoped lang="less">
.chart-wraper {
  height: 100%;
  position: relative;
  .chart {
    position: absolute;
    left: 0;
    top: 0;
    right: 0;
    bottom: 0;
  }
}
</style>
